/***************************************************************************
                          cutescintilla.cpp  -  extension on qscintilla
                             -------------------
    begin                : Sam Jul 19 23:53:49 CEST 2003
    copyright            : (C) 2003 by Heiko Khler
    email                : heicom@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qapplication.h>
#include <qobjectlist.h>

#include "cutescintilla.h"
#include "cute.h"

/* Convert a Qt character to the Scintilla equivalent.  Return zero if it is invalid */
static int convert(int key)
{
	// Convert the modifiers.
	int sci_mod = 0;

	if (key & Qt::SHIFT)
		sci_mod |= QextScintillaBase::SCMOD_SHIFT;

	if (key & Qt::CTRL)
		sci_mod |= QextScintillaBase::SCMOD_CTRL;

	if (key & Qt::ALT)
		sci_mod |= QextScintillaBase::SCMOD_ALT;

	key &= ~Qt::MODIFIER_MASK;

	// Convert the key.
	int sci_key;

	if (key > 0x7f)
		switch (key)
		{
		case Qt::Key_Down:
			sci_key = QextScintillaBase::SCK_DOWN;
			break;

		case Qt::Key_Up:
			sci_key = QextScintillaBase::SCK_UP;
			break;

		case Qt::Key_Left:
			sci_key = QextScintillaBase::SCK_LEFT;
			break;

		case Qt::Key_Right:
			sci_key = QextScintillaBase::SCK_RIGHT;
			break;

		case Qt::Key_Home:
			sci_key = QextScintillaBase::SCK_HOME;
			break;

		case Qt::Key_End:
			sci_key = QextScintillaBase::SCK_END;
			break;

		case Qt::Key_Prior:
			sci_key = QextScintillaBase::SCK_PRIOR;
			break;

		case Qt::Key_Next:
			sci_key = QextScintillaBase::SCK_NEXT;
			break;

		case Qt::Key_Delete:
			sci_key = QextScintillaBase::SCK_DELETE;
			break;

		case Qt::Key_Insert:
			sci_key = QextScintillaBase::SCK_INSERT;
			break;

		case Qt::Key_Escape:
			sci_key = QextScintillaBase::SCK_ESCAPE;
			break;

		case Qt::Key_Backspace:
			sci_key = QextScintillaBase::SCK_BACK;
			break;

		case Qt::Key_Tab:
			sci_key = QextScintillaBase::SCK_TAB;
			break;

		case Qt::Key_Return:
			sci_key = QextScintillaBase::SCK_RETURN;
			break;

		default:
			sci_key = 0;
		}
	else
		sci_key = key;

	if (sci_key)
		sci_key |= (sci_mod << 16);

	return sci_key;
}

CUTEScintilla::CUTEScintilla(QWidget *parent, const char *name) : QextScintilla(parent, name)
{
	CUTE *cute = static_cast<CUTE*>(qApp->mainWidget());

	menu = new QPopupMenu(this);
	QObjectList *childs = queryList();
	for( QObject *obj = childs->first(); obj; obj = childs->next() ){
		if(!strcmp(obj->className(), "QWidget"))
			obj->installEventFilter(this);
	}
	
	cute->editUndo->addTo(menu);
	cute->editRedo->addTo(menu);
	menu->insertSeparator();
	cute->editCut->addTo(menu);
	cute->editCopy->addTo(menu);
	cute->editPaste->addTo(menu);
	cute->editDelete->addTo(menu);
	menu->insertSeparator();
	cute->editSelectAll->addTo(menu);
	menu->insertSeparator();
	menu->insertItem("Bookmarks", cute->bookmarksMenu );
	cute->searchFind->addTo(menu);
	cute->searchFindSelection->addTo(menu);
	cute->searchFindTag->addTo(menu);
	menu->insertSeparator();
	cute->fileClose->addTo(menu);
}

bool CUTEScintilla::eventFilter(QObject *o, QEvent *e)
{
	if(e->type() == QEvent::ContextMenu){
		contextMenu(static_cast<QContextMenuEvent *>(e));
		return true;
	}
	return QextScintilla::eventFilter(o,e);
}

void CUTEScintilla::contextMenu(QContextMenuEvent *e)
{
	static_cast<CUTE*>(qApp->mainWidget())->slotUpdateBookmarksMenu();
	menu->popup(QPoint(e->globalX(), e->globalY()));
}

void CUTEScintilla::setActions(QMap<int, QAction*> map)
{
	int sci_accel;

	SendScintilla(SCI_CLEARALLCMDKEYS);

	for( QMap<int, QAction*>::iterator i = map.begin(); i != map.end(); ++i ){
		sci_accel = convert(i.data()->accel()[0]);
		if(i.data()->accel().count() > 0)
			if(sci_accel != 0){
				SendScintilla(SCI_CLEARCMDKEY, i.key());
				SendScintilla(SCI_ASSIGNCMDKEY, sci_accel, i.key());
			}
	}
}


