/***************************************************************************
                          cuteshell.cpp  -  description
                             -------------------
    begin                : Sam Aug 10 2:5:49 CEST 2002
    copyright            : (C) 2003 by Heiko Khler
    email                : heicom@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
 
#include <stdio.h>
#include <unistd.h> 

#include <qstringlist.h>
#include <qregexp.h>
#include <qcursor.h>

#include "cuteshell.h"

CUTEShell::CUTEShell( QWidget *parent, const char *name )
	: QTextEdit(parent, name)
{
	setWordWrap(NoWrap);
	setFont(QFont("courier", 10));
	connect(this, SIGNAL(returnPressed()), this, SLOT(runCommand()));
	QStringList args = QStringList::split(" ", "/bin/sh");
	proc = new QProcess(args, this);
	connect(proc, SIGNAL(readyReadStdout()), this, SLOT(readStdout()));
	connect(proc, SIGNAL(readyReadStderr()), this, SLOT(readStderr()));
	connect(proc, SIGNAL(processExited()), this, SLOT(processExited()));
	proc->start();
}

void CUTEShell::readStdout()
{
	QString line;
	while(proc->canReadLineStdout()){
		line = proc->readLineStdout();
		setColor(Qt::blue);
		insertParagraph(line, -1);
		setCursorPosition(lines()-1, 0);
	}
	setColor(Qt::black);
}

void CUTEShell::readStderr()
{
	QString line;
	while(proc->canReadLineStderr()){
		line = proc->readLineStderr();
		setColor(Qt::red);
		insertParagraph(line, -1);
		setCursorPosition(lines()-1, 0);
	}
	setColor(Qt::black);
}

void CUTEShell::processExited()
{
	setColor(Qt::red);
	insertParagraph("Shell restarted", -1);
	setCursorPosition(lines()-1, 0);
	setColor(Qt::black);
	proc->start();
}

void CUTEShell::runCommand()
{
	QString line;
	
	line = text(lines()-2);
	proc->writeToStdin(line+"\n");
}

QPopupMenu *CUTEShell::createPopupMenu( const QPoint & pos )
{
	contextMenu = new QPopupMenu(this);
	contextMenu->insertItem("Clear", this, SLOT(clear()));
	contextMenu->insertItem("Kill", proc, SLOT(kill()));
	return contextMenu;
}
