/***************************************************************************
                          cuteview.cpp  -  description
                             -------------------
    begin                : Sam Sep 28 23:53:49 CEST 2002
    copyright            : (C) 2002 by Heiko Khler
    email                : heicom@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qpushbutton.h>
#include <qlayout.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qmessagebox.h>
#include <qfiledialog.h>
#include <qextscintillacommand.h>
#include <qextscintillacommandset.h>

#include "cute.h"
#include "lexer.h"
#include "cuteview.h"
#include "cutescintilla.h"
#include "config.h"

ScintillaAction::ScintillaAction(unsigned int msg, unsigned long wParam, long lParam)
{
	int len_t;
	char *t;

	this->msg = msg;
	this->wParam = wParam;
	t = (char*)lParam;
	if(t != NULL){
		len_t = strlen(t);
		this->lParam = new char[len_t+1];
		strcpy( this->lParam, t );
	}
	else
		this->lParam = NULL;
}

ScintillaAction::ScintillaAction(const ScintillaAction& a)
{
	int len_lParam;

	msg = a.msg;
	wParam = a.wParam;
	if(a.lParam != NULL){
		len_lParam = strlen(a.lParam);
		lParam = new char[len_lParam];
		strcpy(lParam, a.lParam);
	}
	else
		lParam = NULL;
}

ScintillaAction &ScintillaAction::operator=(const ScintillaAction& a) 
{
	int len_lParam;

	msg = a.msg;
	wParam = a.wParam;
	if(a.lParam != NULL){
		len_lParam = strlen(a.lParam);
		lParam = new char[len_lParam];
		strcpy(lParam, a.lParam);
	}
	else
		lParam = NULL;
	return *this;
}

CUTEView::CUTEView(QWidget *parent, QString title) : QextMdiChildView( title, parent)
{
	filename = 0;
	recording = false;
	searching = false;
	setFocusPolicy(QWidget::NoFocus);
	_lexer = Config::LexerManager::lexer("");
	QHBoxLayout *box = new QHBoxLayout(this);
	view = new CUTEScintilla(this);
	connect(view, SIGNAL(marginClicked(int,int,Qt::ButtonState)), 
			this, SLOT(slotMarginClicked(int,int,Qt::ButtonState)));
	connect(view, SIGNAL(SCN_MACRORECORD(unsigned int,unsigned long,long)), 
			this, SLOT(recordMacro(unsigned int,unsigned long,long)));
	connect(view, SIGNAL(SCN_CHARADDED(int)), this, SLOT(slotCharAdded(int)));
	box->addWidget(view);
	view->setMarginLineNumbers(CUTE_MARGIN, false);
	view->setMarginLineNumbers(CUTE_LINENUMBER_MARGIN, true);
	bookmark = view->markerDefine(QextScintilla::Circle);
	view->setMarkerBackgroundColor(Qt::red, bookmark);
	error = view->markerDefine(QextScintilla::RightArrow);
	view->setMarkerBackgroundColor(Qt::yellow, error);
	view->setMarginWidth(CUTE_MARGIN, 16);
	view->setMarginSensitivity(CUTE_MARGIN, 1);
	view->setMarginMarkerMask(CUTE_MARGIN, 1 << bookmark);
	view->setAutoCompletionSource(QextScintilla::AcsDocument);
	view->setAutoCompletionCaseSensitivity(true);
	view->setCallTipsVisible(0);
	configure();
}

void CUTEView::configure()
{
	view->setBackspaceUnindents(Config::edit.getInteger("backspaceUnindents"));
	view->setEolVisibility(Config::view.getInteger("eolVisibility"));
	view->setIndentationsUseTabs(Config::edit.getInteger("indentationsUseTabs"));
    view->setBraceMatching((QextScintilla::BraceMatch)Config::view.getInteger("braceMatching"));
	view->setFolding((QextScintilla::FoldStyle)Config::view.getInteger("folding"));
	view->setTabWidth(Config::edit.getInteger("tabWidth"));
	view->setTabIndents(Config::edit.getInteger("tabIndents"));
	view->setIndentationGuides(Config::view.getInteger("indentationsGuide"));
	view->setAutoCompletionCaseSensitivity(Config::edit.getInteger("autoCompletionCaseSensitivity"));
	QextScintilla::EolMode lineFeed = (QextScintilla::EolMode)Config::edit.getInteger("lineFeed");
	view->setEolMode(lineFeed);
	view->convertEols(lineFeed);
	view->setWhitespaceVisibility((QextScintilla::WhitespaceVisibility)Config::view.getInteger("whitespaces"));
	view->setAutoCompletionThreshold(Config::edit.getInteger("autoCompletionThreshold"));
	if(Config::edit.getInteger("autoIndent"))
		view->SendScintilla( QextScintillaBase::SCI_SETPROPERTY, "indent.automatic", "1");
	view->setAutoIndent(Config::edit.getInteger("autoIndent"));
	if(Config::view.getInteger("lineNumbers"))
		view->setMarginWidth(CUTEView::CUTE_LINENUMBER_MARGIN, "00000");
	else
		view->setMarginWidth(CUTEView::CUTE_LINENUMBER_MARGIN, "");
  
	if(Config::view.getInteger("margin"))
		view->setMarginWidth(CUTEView::CUTE_MARGIN, "00");
	else
		view->setMarginWidth(CUTEView::CUTE_MARGIN, "");

	if(Config::view.getInteger("foldMargin"))        
		view->setMarginWidth(CUTEView::CUTE_FOLD_MARGIN, "00");
	else
		view->setMarginWidth(CUTEView::CUTE_FOLD_MARGIN, "");
	if(Config::view.getInteger("wrapLines"))
		view->SendScintilla(QextScintillaBase::SCI_SETWRAPMODE, QextScintillaBase::SC_WRAP_WORD );
	else
		view->SendScintilla(QextScintillaBase::SCI_SETWRAPMODE, (unsigned long)QextScintillaBase::SC_WRAP_NONE );
}

CUTEView::~CUTEView()
{
	delete view;
}

void CUTEView::sendEditor(unsigned int msg, unsigned long wParam, long lParam)
{
	view->SendScintilla(msg, wParam, lParam);
}

bool CUTEView::edited()
{
	return view->isModified();
}

bool CUTEView::canUndo()
{
	return view->isUndoAvailable();
}

bool CUTEView::canRedo()
{
	return view->isRedoAvailable();
}

void CUTEView::undo()
{
	static_cast<QextScintilla*>(view)->undo();
}

void CUTEView::redo()
{
	static_cast<QextScintilla*>(view)->redo();
}

QextScintilla* CUTEView::scintilla()
{
	return view;
}

void CUTEView::setActions( QMap<int, QAction*> map )
{
	view->setActions(map);
}

bool CUTEView::loadFile(const char* name)
{
	qWarning(name);
	QString line;
	if(!name)
		return false;
	filename = new char[strlen(name)+1];
	strcpy(filename, name);
	QFile file(name);
	
	if( !file.open(IO_ReadOnly))
		return false;
	QTextStream in(&file);
	QString text;
	while( !in.atEnd() ){
		line = in.readLine();
		text += line+"\n";
	}
	file.close();
	view->setText(text);
	return true;
}

bool CUTEView::reloadFile()
{
	int line, col;
	bool ret;
	view->getCursorPosition( &line, &col );
	static_cast<QextScintilla*>(view)->clear();
	ret = loadFile(filename);
	view->setCursorPosition( line, col );
	return ret;
}

bool CUTEView::saveFile(const char *fn, bool force)
{
	QString filename;

	if(!view->isModified() && view->length() && !force)
		return true;

	if( fn ){
		this->filename = new char[strlen(fn)+1];
		strcpy(this->filename, fn);
	}

	filename = this->filename;
	qWarning(filename);
	setCaption(filename);
	setTabCaption(filename);
	setMDICaption(filename);
	QFile file(filename);
	if( !file.open(IO_WriteOnly))
		return false;
	QTextStream out(&file);
	out << view->text();
	file.close();
	view->SendScintilla(QextScintillaBase::SCI_SETSAVEPOINT);
	return true;
}

void CUTEView::toggleBookmark(int lineno)
{
	int state = view->SendScintilla(QextScintillaBase::SCI_MARKERGET, lineno);
	CUTE* cute = static_cast<CUTE*>(qApp->mainWidget());
	if ( state & (1 << bookmark)) {
		view->SendScintilla(QextScintillaBase::SCI_MARKERDELETE, lineno, bookmark);
		cute->deleteBookmark();
	} else {
		int handle = view->SendScintilla(QextScintillaBase::SCI_MARKERADD, lineno, bookmark);
		cute->addBookmark(new Bookmark(this, handle));
	}
}

void CUTEView::addError( int lineno )
{
	int handle = view->SendScintilla(QextScintillaBase::SCI_MARKERADD, lineno, error);
}

void CUTEView::deleteAllErrors()
{
	view->SendScintilla(QextScintillaBase::SCI_MARKERDELETEALL, error);
}

void CUTEView::findNextBookmark()
{
	int lineno, index;
	view->getCursorPosition(&lineno, &index);
	int nextLine = view->SendScintilla(QextScintillaBase::SCI_MARKERNEXT, lineno + 1, 1 << bookmark);
	if (nextLine < 0)
		nextLine = view->SendScintilla(QextScintillaBase::SCI_MARKERNEXT, lineno, 1 << bookmark);
	if (nextLine < 0 || nextLine == lineno)	// No bookmark (of the given type) or only one, and already on it
		QMessageBox::warning(0,0,"No bookmark found");
	else {
		view->SendScintilla(QextScintillaBase::SCI_GOTOLINE, nextLine);
	}
}

void CUTEView::findPrevBookmark()
{
	int lineno, index;
	view->getCursorPosition(&lineno, &index);
	int prevLine = view->SendScintilla(QextScintillaBase::SCI_MARKERPREVIOUS, lineno - 1, 1 << bookmark);
	if (prevLine < 0)
		prevLine = view->SendScintilla(QextScintillaBase::SCI_MARKERPREVIOUS, lineno, 1 << bookmark);
	if (prevLine < 0 || prevLine == lineno) {	// No bookmark (of the given type) or only one, and already on it
		QMessageBox::warning(0,0,"No bookmark found");
	} else {
		view->SendScintilla(QextScintillaBase::SCI_GOTOLINE, prevLine);
	}
}

void CUTEView::closeEvent(QCloseEvent *e)
{
	if(view->isModified()){
		switch( QMessageBox::warning(this, "CUTE", "Document modified. Would you like to save it?", "Yes", "No", "Cancel") ){
		  case 0:
			if(filename)
				saveFile();
			else
				emit save();
			if(!view->isModified()){
				removeAllBookmarks();
				QextMdiChildView::closeEvent(e);
			}
			break;
		  case 1:
			removeAllBookmarks();
			QextMdiChildView::closeEvent(e);
			break;
		  case 2:
			break;
		}
	}
	else{
		removeAllBookmarks();
		QextMdiChildView::closeEvent(e);
	}
}

void CUTEView::slotDocumentChanged()
{
  //TODO update the view
}

void CUTEView::slotMarginClicked(int margin, int line, Qt::ButtonState state)
{
	if(margin == CUTE_MARGIN)
		toggleBookmark(line);
}

void CUTEView::slotCharAdded(int)
{
	deleteAllErrors();
}

void CUTEView::removeAllBookmarks()
{
	CUTE* cute = static_cast<CUTE*>(qApp->mainWidget());
	for(int i = 0; i < view->lines(); i++){
		view->SendScintilla(QextScintillaBase::SCI_MARKERDELETE, i, bookmark);
		cute->deleteBookmark();
	}
}

void CUTEView::insertCommand(QString cmd)
{
	view->beginUndoAction();
	proc = new QProcess( QStringList::split(QRegExp("\\s"), cmd ), this );
	connect( proc, SIGNAL(readyReadStdout()), this, SLOT(insertStdout()));
	proc->start();
	view->endUndoAction();
}

void CUTEView::filter(QString cmd)
{
	QString text;
	QString fileName;
	
	// creates temporare file
	fileName = QDir::homeDirPath()+QDir::separator()+".cute/.cute.tmp";
	QFile tempFile(fileName);
	
	// writes selection to file
	tempFile.open(IO_WriteOnly);
	QTextStream str(&tempFile);
	text = view->selectedText();
	str << text;
	tempFile.close();
	view->beginUndoAction();
	view->removeSelectedText();
	
	// start shell, executing pipe command cat cute.tmp | cmd
	proc = new QProcess(QString("/bin/sh"));
	proc->addArgument("-c");
	proc->addArgument("cat " + fileName + " | " + cmd);
	connect( proc, SIGNAL(readyReadStdout()), this, SLOT(insertStdout()));
	connect( proc, SIGNAL(processExited()), this, SLOT(processExited()));
	proc->start();
}

void CUTEView::insertStdout()
{
	int lineno, index;
	QString line;
	
	while(proc->canReadLineStdout()) {
		line = proc->readLineStdout();
		line += "\n";
		view->getCursorPosition(&lineno, &index);
		view->insertAt(line, lineno, index);
		view->setCursorPosition(lineno+1, 0);
	}
}

void CUTEView::processExited()
{
	QString fileName = QDir::homeDirPath()+QDir::separator()+".cute/.cute.tmp";
	QFile file(fileName);
	
	file.remove();
	view->endUndoAction();
	delete proc;
}

void CUTEView::startMacroRecording()
{
	if(recording){
		switch(QMessageBox::warning(this, "CUTE", "Macro recording is activated", "New Start", "Cancel"))
		{
		case 0:
			stopMacroRecording();
			break;
		case 1:
			return;
		default:
			qWarning("internal error in start macro recording dialog");
		};
	}
	else
		recording = true;
	macro.clear();
	view->SendScintilla(QextScintillaBase::SCI_STARTRECORD);
}

void CUTEView::stopMacroRecording()
{
	if(!recording){
		QMessageBox::warning(this, "CUTE", "No macro recording");
		return;
	}
	recording = false;
	view->SendScintilla(QextScintillaBase::SCI_STOPRECORD);
}

void CUTEView::runMacro()
{
	if(recording){
		switch(QMessageBox::warning(this, "CUTE", "Macro recording is activated", "Stop Recording", "Cancel"))
		{
		case 0:
			stopMacroRecording();
			break;
		case 1:
			return;
		default:
			qWarning("internal error in run macro dialog");
		};
	}
	view->beginUndoAction();
	std::list<ScintillaAction>::iterator i = macro.begin();
	for(; i != macro.end(); ++i)
		view->SendScintilla(i->msg, i->wParam, i->lParam);
	view->endUndoAction();
}

void CUTEView::loadMacro(QString &file)
{
	QFile f(file);
	QString macro_str;
	
	if(!f.open(IO_ReadOnly)){
		QMessageBox::warning(this, "CUTE", "Could not open file");
		return;
	}
	QTextStream str(&f);
	macro_str = str.read();
	macro.clear();
	if( !setMacro(macro_str) ){
		QMessageBox::warning(this, "CUTE", "Erro while loading macro");
		return;
	}
	f.close();
}

void CUTEView::saveMacro(QString &file)
{
	QFile f(file);
	
	if(!f.open(IO_WriteOnly)){
		QMessageBox::warning(this, "CUTE", "Could not open file");
		return;
	}
	if(!macroString()){
		QMessageBox::information(this, "CUTE", "No macro recorded");
		return;
	}
	QTextStream str(&f);
	str << macroString();
	f.close();
}

bool CUTEView::setMacro(QString macro_str)
{
	QTextStream str(&macro_str, IO_ReadOnly);
	QString lParam_str;
	unsigned int msg;
	unsigned long wParam;
	long lParam;
	bool ok;

	macro.clear();
	while(!str.atEnd()){
		str >> msg >> wParam >> lParam_str;
		lParam = reinterpret_cast<long>(lParam_str.latin1());
		recordMacro(msg, wParam, lParam);
	}
	return true;
}

QString CUTEView::macroString()
{
	QString macro_str;
	QTextStream str(&macro_str, IO_WriteOnly);
	
	std::list<ScintillaAction>::iterator i = macro.begin();
	for(; i != macro.end(); ++i){
		str << i->msg << " " << i->wParam << " " << i->lParam << endl;
	}
	return macro_str;
}

void CUTEView::recordMacro(unsigned int msg, unsigned long wParam, long lParam)
{
	ScintillaAction action(msg, wParam, lParam);

	macro.push_back(action);
}

bool CUTEView::findFirst(const QString &expr, bool re, bool cs, bool wo, bool wrap, bool forward, int line, int index)
{
	searchString = expr;
	searchRegExp = re;
	searchCaseSensitive = cs;
	searchWholeWord = wo;
	searchLineWrap = wrap;
	searchDirection = forward;
	searching = true;
	return view->findFirst(expr, re, cs, wo, wrap, forward, line, index);
}

bool CUTEView::findNext()
{
	int line, index;

	if(!searching)
		return false;
	view->getCursorPosition(&line, &index);
	return view->findFirst( searchString, searchRegExp, searchCaseSensitive, searchWholeWord, searchLineWrap, 
		searchDirection, line, index);
}

bool CUTEView::findPrev()
{
	int line, index;

	if(!searching)
		return false;
	view->getCursorPosition(&line, &index);
	if(searchDirection)
		index -= searchString.length();
	view->findFirst( searchString, searchRegExp, searchCaseSensitive, searchWholeWord, searchLineWrap, 
		!searchDirection, line, index);
	return view->findNext();
}

QString CUTEView::line(int line)
{
	QString str = view->text(line);
	int len = view->SendScintilla(QextScintillaBase::SCI_GETLINEENDPOSITION, line) - 
		view->SendScintilla(QextScintillaBase::SCI_POSITIONFROMLINE, line);
	return str.left(len);
}

void CUTEView::setupLexer(const QString &fileName)
{
	/* choose lexer */
	_lexer = Config::LexerManager::lexer(fileName);
	if( _lexer )
		_lexer->initScintilla(view);
}

void CUTEView::replaceAll(const QString &expr, const QString &replaceStr, bool re, bool cs, bool wo, bool wrap, bool foreward, bool confirm)
{
	bool found;
	int prevLine, prevIndex;
	int line, index;

	found = findFirst(expr,
						re,
						cs,
						wo,
						wrap,
						foreward, 0, 0);
	while(found){
		if(confirm)
			switch( QMessageBox::information(qApp->mainWidget(), "CUTE", "Replace String?", "Replace", "Omitt","Cancel") )
			{
			case 0:
				view->replace(replaceStr);
				break;
			case 1:
				break;
			case 2:
				return;
			}
		else
			view->replace(replaceStr);
		view->getCursorPosition(&prevLine, &prevIndex);
		found = findNext();
		view->getCursorPosition(&line, &index);
		if( line < prevLine)
			return;
		else if( line == prevLine && index <= prevIndex )
			return;
	}
}
